<?php

class NoteController
{
    private $cnmNames = Array();
    private $cnmDirs = Array();
    private $toc = Array();
    private $cnName = "";
    private $cnTitle = "";

    public function index($root)
    {
        // Menu
        // 'cnmdirs' = Directory for courses that contain
        //             course notes (main menu)
        $this->cnmdirs = $this->scan_subdirs($root.'/modl/');
        for($i = 0; $i < count($this->cnmdirs); $i++)
        {
            // The info.txt file should contain only two lines.
            $tmp = $this->scan_infotxt($root.'/modl/'.$this->cnmdirs[$i]);
            if($tmp['length'] == 3)
                $this->cnmNames[$i] = $tmp;
        }
        $pageTitle = 'List of Course Notes';
        $hasStyle = true;
        $stylesheet = file_get_contents($root.'/view/shared/css/index.css');
        include_once $root.'/view/shared/_layout_t.php';
        include_once $root.'/view/shared/index.php';
        include_once $root.'/view/shared/_layout_b.php';
    }
    
    public function getContent($root,$url)
    {
        // Get the root URL and name of the course note.
        $rootURL = explode('/', filter_var(rtrim($_GET['url'], '/'),
                           FILTER_SANITIZE_URL))[0];
        $pageFound = true; // info.txt must reside in every folder below /modl.
         
        // Main Menu
        $this->cnmdirs = $this->scan_subdirs($root.'/modl/');
        for($i = 0; $i < count($this->cnmdirs); $i++)
        {
            $this->cnmNames[$i] = $this->scan_infotxt(
                $root.'/modl/'.$this->cnmdirs[$i]);
            if($this->cnmdirs[$i] == $rootURL)
                $this->cnTitle = $this->cnmNames[$i]['title'];
        }
        if(file_exists($root.'/modl'.$url.'/index.html'))
        {
            $this->cnName = $this->scan_infotxt($root.'/modl/'.$url);
            $this->displayPage($root,
                $this->cnName['title'],true,
                file_get_contents($root.'/view/shared/css/note.css'),
                '/modl'.$url.'/index.html',true);
        }
        else
        {
            // Sections containing index.html and/or info.txt
            // $toc = Table of content
            $this->toc = $this->dirToArray($root.'/modl'.$url,$root);
        
            $this->cnName = $this->scan_infotxt($root.'/modl/'.$url);
            
            // If the URL is invalid or there is no info.txt in the main
            // folder...
            if($this->toc[0] == "error")
            {
                $this->displayPage($root,'Error',false,'',
                    '/view/shared/error.php');
            }
            else if($this->toc[0] == "error-infotxt"
                || empty($this->cnName['title']))
            {
                $this->displayPage($root,
                    'Error: Malformed <code>info.txt</code>.',
                    false,'','/view/shared/error_infotxt.php');
            }
            else if(count($this->toc) == 0) // No subfolders?
            {
                $this->displayPage($root,'Please Add Subfolders',false,'',
                    '/view/shared/toc-addfolder.php');
            }
            else // Show the table of content.
            {
                $this->displayPage($root,'Table of Content',false,'',
                    '/view/shared/toc.php');
            }
        }
    }
    
    public function api_getContent($root,$url)
    {
        echo 'API called!<br />';
    }

    public function error($root)
    {
        $this->cnmdirs = $this->scan_subdirs($root.'/modl/');
        for($i = 0; $i < count($this->cnmdirs); $i++)
        {
            $this->cnmNames[$i] = $this->scan_infotxt(
                $root.'/modl/'.$this->cnmdirs[$i]);
        }
        http_response_code(404);
        $this->displayPage($root,'Error',false,'','/view/shared/error-404.php');
    }

    function displayPage($root,$_pageTitle,$_hasStyle,$_stylesheet,$page,
                         $isNote = false)
    {
       $pageTitle = $_pageTitle;
       $hasStyle = $_hasStyle;
       $stylesheet = $_stylesheet;
       include_once $root.'/view/shared/_layout_t.php';
       include_once $root.$page;
       if($isNote) include_once $root.'/view/shared/_layout_b_note.php';
       else include_once $root.'/view/shared/_layout_b.php';
    }

    function scan_subdirs($dir)
    {
        $sub_dirs = array();
        $dir_to_check = scandir($dir);
        foreach ($dir_to_check as $item)
            if ($item != '..' && $item != '.' && is_dir($dir .
                DIRECTORY_SEPARATOR . $item)) {
                // Make sure there is an info.txt file before pushing the $item
                // into the array.
                if(file_exists($dir . $item . DIRECTORY_SEPARATOR
                    . 'info.txt'))
                    if($this->scan_infotxt($dir.$item)['length'] == 3)
                        array_push($sub_dirs, $item);
            }
        return $sub_dirs;
    }
    
    function getDirContents($root, $includeFileName, &$results = array()) {
        $files = scandir($root);
        foreach ($files as $key => $value) {
            $path = realpath($root.'/'.$value);
            if (!is_dir($path)) {
                if($includeFileName)
                    $results[] = $path;
            } else if ($value != "." && $value != "..") {
                $this->getDirContents($path, $includeFileName, $results);
                if(!$includeFileName)
                    $results[] = $path;
            }
        }
        return $results;
    }
    
    // https://stackoverflow.com/questions/952263/deep-recursive-array-of-directory-structure-in-php
    function dirToArray($dir,$root) {
        $contents = array();
        $link = "#";
        # Foreach node in $dir
        foreach (scandir($dir) as $node) {
            # Skip link to current and parent folder
            if ($node == '.')  continue;
            if ($node == '..') continue;
            # Check if it's a node or a folder
            if (is_dir($dir . DIRECTORY_SEPARATOR . $node)) {
                if(file_exists($dir . DIRECTORY_SEPARATOR . $node . '/index.html'))
                {
                    $rootLengthView = strlen($root.'/modl');
                    $link = substr($dir, $rootLengthView, strlen($dir)).'/'.$node;
                }
                if(file_exists($dir . DIRECTORY_SEPARATOR . $node . '/info.txt'))
                {
                    $info = $this->scan_infotxt($dir.'/'.$node);
                    if($info["length"] == 3)
                    {
                        $infoWithLink = [
                            "title" => $info['title'],
                            "descr" => $info['descr'],
                            "length" => $info['length'],
                            "link"  => $link
                        ];
                    }
                    else return ["error-infotxt"];
                } else return ["error"];
                // Add directory recursively, be sure to pass a valid path
                // to the function, not just the folder's name
                $temp = $this->dirToArray($dir . DIRECTORY_SEPARATOR . $node,
                                          $root);
                If(count($temp) > 0) {
                    $contents[] = [$node, $infoWithLink, $temp];
                } else $contents[] = [$node, $infoWithLink];
            }
        }
        # done
        return $contents;
    }
    
    function scan_infotxt($dir)
    {
        $file = $dir.'/info.txt';
        $array = explode(PHP_EOL, file_get_contents($file));
        $json['title'] = $array[0];
        $json['descr'] = $array[1];
        $json['length'] = count($array);
        return $json;
    }
}
